﻿define([
    'dojo/_base/declare',
    'dijit/_WidgetBase',
    'dijit/_TemplatedMixin',
    'dojo/text!./Templates/AddToBasketGadget.html',
    'epi-cms/_ContentContextMixin',
    'dijit/_WidgetsInTemplateMixin',
    'dijit/form/Button',
    'epi/i18n!./nls/localization.LWEPIServerConnector'
], function (
    declare,
    _WidgetBase,
    _TemplatedMixin,
    template,
    _ContentContextMixin,
    _WidgetsInTemplateMixin,
    button,
    translations
) {
    var $ = window.epiJQuery;

    return declare('languagewire.translation.AddToBasketGadget',
    [_WidgetBase, _TemplatedMixin, _ContentContextMixin, _WidgetsInTemplateMixin], {

        translations: translations,
        activeDialog: null,
        windowHeight: 600,
        windowWidth: 960,
        templateString: template,

        constructor: function () {
            var that = this;
            $(window).resize(function () {
                that.onResize();
            });
            this.initializePanels();
        },

        contextChanged: function () {
            this.initializePanels();
        },

        getRelativeBaseUrl: function () {
            var url = window.location.pathname;
            url = url.replace(/\/cms/gi, ''); // remove "/CMS"
            url = url + '/LanguageWire.Connector';
            return url;
        },

        resizeActiveDialog: function () {
            this.setDialogSize(this.activeDialog, this.windowHeight, this.windowWidth);
        },

        openActiveDialog: function (url) {
            var that = this;
            this.activeDialog = $('#lwDialog');
            this.activeDialog.html('<iframe src="' + url + '" height="' + that.windowHeight + 'px" width="' + that.windowWidth + 'px"></iframe>');
            this.activeDialog.dialog({
                autoOpen: false,
                height: that.windowHeight,
                width: that.windowWidth,
                modal: true,
                position: { my: 'center', at: 'center', of: window },
                resizable: false,
                open: function() {
                    that.resizeActiveDialog();
                },
                close: function (arg) {
                    if (arg.bubbles) {
                        that.closeActiveDialog();
                    }
                }
            });

            this.activeDialog.dialog('open');
        },

        closeActiveDialog: function () {
            if (this.activeDialog) {
                this.activeDialog.dialog('close');
                this.activeDialog = null;
                this.initializePanels();
            }
        },

        onResize: function () {
            if (this.activeDialog)
                this.setDialogSize(this.activeDialog, this.windowHeight, this.windowWidth);
        },

        setDialogSize: function (dialog, height, width) {
            const wHeight = $(window).height();
            const wWidth = $(window).width();

            if (height > wHeight) {
                $(dialog).css('height', wHeight);
                $(dialog).css('overflow-y', 'scroll');
            } else {
                $(dialog).css('height', height);
                $(dialog).css('overflow-y', 'hidden');
            }

            if (width > wWidth) {
                $(dialog).css('width', wWidth);
                $(dialog).css('overflow-x', 'scroll');
            } else {
                $(dialog).css('width', width);
                $(dialog).css('overflow-x', 'hidden');
            }
        },


        checkIfPageHasDescendents: function (id, language) {
            var that = this;
            $.ajax({
                type: 'POST',
                contentType: 'application/json; charset=utf-8',
                cache: false,
                async: true,
                url: that.getRelativeBaseUrl() + '/Services/Gadget.asmx/CheckPageSubitems',
                dataType: 'json',
                data: JSON.stringify({ pageId: id, language: language }),
                success: function (data) {
                    if (data.d === 'false') {
                        that.addSubBtn.setDisabled(true);
                    }
                    else {
                        that.addSubBtn.setDisabled(false);
                    }
                },
                error: function () {
                }
            });
        },

        updatePanels: function (itemsCount, basketLanguage) {
            const ctx = this.getCurrentContext();
            var basketIsEmpty = true;

            if (itemsCount > 0) {
                this.emptyBasket.style.display = 'none';
                this.nonEmptyBasket.style.display = 'block';
                this.basketCount.innerHTML = itemsCount;
                this.basketLanguage.innerHTML = basketLanguage;
                this.setBasketLanguageIcon(basketLanguage);
                this.clearBasketBtn.setDisabled(false);
                this.basketBtn.setDisabled(false);
                basketIsEmpty = false;
            } else {
                this.emptyBasket.style.display = 'block';
                this.nonEmptyBasket.style.display = 'none';
                this.clearBasketBtn.setDisabled(true);
                this.basketBtn.setDisabled(true);
            }

            const pageIsRoot = (ctx.hasTemplate === false && ctx.dataType === 'episerver.core.pagedata');
            const preferredLanguage = ctx.languageContext !== null ? ctx.languageContext.preferredLanguage : null;
            const basketContainsItemsInOtherLanguage = !basketIsEmpty && basketLanguage !== preferredLanguage;
            var addButtonDisabled = pageIsRoot || basketContainsItemsInOtherLanguage || preferredLanguage !== ctx.language;
            var addWithDescendantsButtonDisabled = pageIsRoot || basketContainsItemsInOtherLanguage;

            if (this.addBtn) {
                this.addBtn.setDisabled(addButtonDisabled);
            }

            if (this.addSubBtn) {
                this.addSubBtn.setDisabled(addWithDescendantsButtonDisabled);
            }

            if (!addWithDescendantsButtonDisabled) {
                this.checkIfPageHasDescendents(ctx.id, preferredLanguage);
            }

            var isSmartAddDisabled = pageIsRoot || basketContainsItemsInOtherLanguage;
            this.smartAddBtn.setDisabled(isSmartAddDisabled);
        },

        // Update basket status panel and buttons by fetching the current basket via webservice
        initializePanels: function () {
            var that = this;
            $.ajax({
                type: 'POST',
                cache: false,
                async: true,
                data: '{}',
                contentType: 'application/json; charset=utf-8',
                dataType: 'json',
                url: that.getRelativeBaseUrl() + '/Services/Gadget.asmx/GetCurrentBasket',
                success: function (data) {
                    const basket = JSON.parse(data.d);
                    const ctx = that.getCurrentContext();
                    that.contextPanel.innerHTML = ctx.name;
                    that.updatePanels(basket.ItemCount, basket.Language);
                },
                error: function () {
                }
            });
        },

        setBasketLanguageIcon: function (language) {
            var that = this;
            $.ajax({
                type: 'POST',
                contentType: 'application/json; charset=utf-8',
                cache: false,
                async: true,
                url: that.getRelativeBaseUrl() + '/Services/Gadget.asmx/GetLanguageIconUrl',
                data: JSON.stringify({ languageName: language }),
                dataType: 'json',

                success: function (data) {
                    const url = data.d;
                    that.basketLanguageIcon.innerHTML = '';
                    if (url) {
                        that.basketLanguageIcon.innerHTML = '<img src="' + url + '" />';
                    }
                },

                error: function () {
                }
            });
        },

        openTransactionDialog: function (withSubItems) {
            var that = this;
            const ctx = this.getCurrentContext();
            const contentId = ctx.id;
            var url = this.getRelativeBaseUrl() + '/app/index.html#add-to-basket';
            const withChildren = withSubItems || false;

            window.gadget = this;
            $.ajax({
                type: 'POST',
                contentType: 'application/json; charset=utf-8',
                cache: false,
                async: true,
                data: JSON.stringify({ contentId: contentId, withChildren: withChildren, language: ctx.languageContext.preferredLanguage }),
                url: that.getRelativeBaseUrl() + '/Services/Gadget.asmx/TryAddBasketItems',
                success: function () {
                    that.openActiveDialog(url);
                },
                error: function (data) {
                    const message = JSON.parse(data.response.match(/({"d":.*})/)[1]).d;
                    alert(message);
                    console.error(message);
                }
            });
        },

        openBasketDialog: function () {
            window.gadget = this;
            const url = this.getRelativeBaseUrl() + '/app/index.html#checkout';
            this.openActiveDialog(url);
        },

        openProjectsDialog: function () {
            window.gadget = this;
            const url = this.getRelativeBaseUrl() + '/app/index.html#projects';
            this.openActiveDialog(url);
        },

        openSmartAddDialog: function () {
            var that = this;
            const ctx = this.getCurrentContext();
            window.gadget = this;
            const smartAddUrl = this.getRelativeBaseUrl() + '/app/index.html#smart-add?id=' + ctx.id + '&language=' + ctx.languageContext.preferredLanguage;

            $.ajax({
                type: 'POST',
                contentType: 'application/x-www-form-urlencoded; charset=utf-8',
                cache: false,
                async: true,
                data: 'language=' + ctx.language,
                url: that.getRelativeBaseUrl() + '/Services/Settings.asmx/CheckConnectorConfiguration',
                success: function () {
                    that.openActiveDialog(smartAddUrl);
        },
                error: function (data) {
                    const configurationStatus = JSON.parse(data.response);
                    alert(configurationStatus.errorMessage);
                    console.error(configurationStatus.errorMessage);
                }
            });
        },

        _onAddItemClick: function () {
            this.openTransactionDialog();
        },

        _onAddWithSubItemsClick: function () {
            this.openTransactionDialog(true);
        },

        _onProjectsClick: function () {
            this.openProjectsDialog();
        },

        _onSmartAddClick: function () {
            this.openSmartAddDialog();
        },

        _onBasketClick: function () {
            this.openBasketDialog();
        },

        _onClearBasketClick: function () {
            const msg = translations.widget.confirmclear;
            if (confirm(msg)) {
                var that = this;
                that.clearBasketBtn.setDisabled(true);
                that.basketBtn.setDisabled(true);
                $.ajax({
                    type: 'POST',
                    cache: false,
                    async: true,
                    data: '{}',
                    contentType: 'application/json; charset=utf-8',
                    dataType: 'json',
                    url: that.getRelativeBaseUrl() + '/Services/Gadget.asmx/ClearBasketSession',
                    success: function () {
                        that.updatePanels(0,'');
                    },
                    error: function () {
                        that.clearBasketBtn.setDisabled(false);
                        that.basketBtn.setDisabled(false);
                    }
                });
            }
        }
    });
});